#!/usr/bin/env python

# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html

import httplib
import os
import re
import signal
import string
import sys
import urlparse
import xml.sax
import xml.sax.handler

""" 
    Script to connect to the R-GMA Services (Consumer, Registry etc). The 
    status of the service is printed to stdout along with a status code as
    follows:
    
    0: Service up
    1: Warning, service maybe running, but a fault developed
    2: Fault, service is down
    3: Unknown, service script failed while running
    
    Status messages are sent to stdout, this includes any error messages.
    
    Usage: rgma-servlet-status [Service-Name]
    
"""
class ServletStatus(xml.sax.ContentHandler):
    
    """ Main class that implements the status commands used by
        the servlet-monitor and ServiceStatusPublisher. """
        
    # Constants
    STATUS = 'Status'
    FREE_SPACE = 'Approx Free Memory (in MBs)'
    TOTAL_SPACE = 'Total Amount of Memory in JVM (in MBs)'
    EXCEPTION = ''
    RGMA_CONF = 'etc/rgma/rgma.conf'
    RGMA_SERVER_AUTH = "etc/rgma/ClientAuthentication.props"
    
    def __init__(self, rgmaHome, serviceName):
        
        """ Attempts to find the url of the service to query. The url is 
            normally derived from the rgma.conf file, however, an entry for the
            BrowserService does not exist within this file. To work round
            this problem, an existing url in the conf file is used with its
            name replaced to contact the correct service (in this case
            the consumer url is used). """
        
        self.currentName = ''
        self.allStatusInfo = {}
        self.foundName = 0
        self.auth = os.path.join(rgmaHome, self.RGMA_SERVER_AUTH)
        rgmaConf = os.path.join(rgmaHome, self.RGMA_CONF)
        propName = re.split('Service$', serviceName)
        if propName[0] == 'Browser':
            self.url = self.__getValue(rgmaConf, 'Consumer')
            if self.url == None:
                sys.stdout.write('Unable to find suitable url to contact: ' + \
                    serviceName + '\n')
                sys.exit(3)
            self.url.replace('Consumer', 'Browser')
        else:
            self.url = self.__getValue(rgmaConf, propName[0])
        if self.url == None:
            sys.stdout.write('ServiceName: ' + serviceName + ' not found\n')
            sys.exit(3)
        self.securemode = self.__isSecure()
        
    def __getValue(self, filename, keyword):
        
        """ Gets the property defined in the rgma.conf file. """
        
        contents = None
        try:
            file = open(filename)
            for line in file.readlines():
                if line != '':
                    bits = line.split("=")
                    if bits[0].strip() == keyword:
                        bits = bits[1].split("#")
                        contents = bits[0].strip()
                        break
            file.close()
        except IOError, e:
            sys.stdout.write(e.__str__() + '\n')
            sys.exit(3)
        return contents
        
    def __isSecure(self):
        
        """ Checks if the url protocol is using ssl. """
        
        bits = urlparse.urlparse(self.url)
        protocol = bits[0]
        if protocol == 'https':
            return True
        else:
            return False
        
    def __getAuthProperties(self):
        
        """ Returns a tuple (sslKeyFile, sslCertFile) where each element is
            a filename. Properties are used to authenticate a client when 
            using ssl. """
        
        sslCertFile = self.__getValue(self.auth, "sslCertFile")
        sslKeyFile = self.__getValue(self.auth, "sslKey")
        if sslCertFile == None or sslKeyFile == None:
            print 'Server authorization file: ' + self.auth + \
                ' is invalid, sslCertFile and sslKey must be set.'
            sys.exit(3)
        if not os.path.isfile(sslCertFile) or not os.path.isfile(sslKeyFile):
            print 'Certificate and key files: [' + sslCertFile + '], [' + \
                sslKeyFile + '] must both exist.'
            sys.exit(3)
        return (sslKeyFile, sslCertFile)
        
    def connect(self, serviceName):
        
        """ Attempts to make a connection to the service using its URL. The 
            script terminates if unable to make a connection. """
        
        # Set signal with a 60 second alarm - to prevent hanging connections
        signal.signal(signal.SIGALRM, urlHandler)
        signal.alarm(60)
        doc = None
        try:
            bits = urlparse.urlparse(self.url)
            hostport = bits[1]
            if self.securemode:
                props = self.__getAuthProperties()
                conn = httplib.HTTPSConnection(hostport, None, \
                   key_file=props[0], \
                   cert_file=props[1])
            else:
                conn = httplib.HTTPConnection(hostport)
            headers = {"Accept": "*/*"}
            conn.request("GET", self.url + '/getNewStatus', None, headers)
            response = conn.getresponse()
            if response.status != 200:
                print 'Error contacting service: %s \n' % response.reason
                sys.exit(1)
            doc = response.read()
            conn.close()
        except httplib.HTTPException, e:
            print 'Unable to connect to URL: %s due to: %s' % (self.url, e)
            sys.exit(2)
        # Disable the alarm
        signal.alarm(0)
        memoryUsage = self.__parse(doc)
        return 'Service running, current memory usage at: %d%% \n' % \
           memoryUsage
        
    def __checkForException(self):
        
        """ Checks if the message was valid or whether an
            Exception was thrown while contacting the servlet. """
        
        if not self.allStatusInfo.has_key(self.STATUS):
            if self.allStatusInfo.has_key(self.EXCEPTION):
                error = self.allStatusInfo.get(self.EXCEPTION)
                print 'Received Exception while contacting servlet: ' + \
                    error
                sys.exit(1)
            else:
                print 'Received Exception while contacting servlet, no ' + \
                    'message available'
                sys.exit(1)
    
    def __parse(self, message):
        
        """ Parses the returned XML and returns a suitable message. """
        
        xml.sax.parseString(message, self)
        self.__checkForException()
        return self.__getMemoryStatus()
        
    def __getMemoryStatus(self):
        
        """ Gets memory usage from getNewStatus and returns
            this as a percentage of space used. """
        
        ratio = None
        try:
            freeSpace = float(self.allStatusInfo.get(self.FREE_SPACE))
            totalSpace = float(self.allStatusInfo.get(self.TOTAL_SPACE))
            ratio = (freeSpace / totalSpace) * 100
        except Exception, e:
            print 'Unable to derive memory usage: %s' % e
            sys.exit(1)
        return ratio
    
    def characters(self, ch):
        
        """ Overides super-type to do something usefull when 
            text content has been found. """
        
        stripped = string.strip(ch)
        if stripped != '':
            if self.foundName:
                self.currentName = ch
            else:
                self.allStatusInfo[self.currentName] = ch
        self.foundName = 0
        
    def startElement(self, name, attrs):
        
        """ Overides super-type. Called when a new xml element
            is found. """
            
        if name == 'name':
            self.foundName = 1
        
def urlHandler(signum, frame):
    
    """ If a getNewStatus takes too long (excess of 60 secs) then an alarm is 
        set and this method will catch it. """
    
    print 'Status request has exceeded time out period of 60 seconds'
    sys.exit(2)
    
def printBanner():
    
    sys.stdout.write('\n')
    sys.stdout.write("Usage:\n\n")
    sys.stdout.write("rgma-servlet-status.py [Service name]\n\n")
    sys.stdout.write("\tChecks the status of specified service\n\n")
    
if __name__ == "__main__":
    
    try:
        rgmaHome = os.environ['RGMA_HOME']
    except KeyError:
        try:
            rgmaHome = os.environ['GLITE_LOCATION']
        except KeyError:
            print 'ERROR: Environment variable RGMA_HOME or ' + \
                'GLITE_LOCATION must be set'
            sys.exit(3)
    length = len(sys.argv)
    if length != 2:
        printBanner()
        sys.exit(3)
    serviceName = sys.argv[1]
    checker = ServletStatus(rgmaHome, serviceName)
    print checker.connect(serviceName)
    sys.exit(0)
    

