#!/usr/bin/env python

# rgma-server-setup
#
#This script configures an RGMA server
#
#It generates an rgma-server.conf file based on which services have been
#requested by command line options. Following this, it generates SQL commands
#to create the necessary databases and if a schema is being configured it
#appends further SQL commands to create schema tables from installed .schema
#files. Finally, if servicetool is installed it arranges for RGMA services
#and their status to be periodically published.

import sys, os, socket, getopt, shutil, string

# Print usage information

def version():

    print "rgma-server-setup version 5.0.31"
    
def usage():

    print("Usage: rgma-server-setup [ --help ] [ --version ]")
    print("                         [ --producerdb=<url> ]")
    print("                         [ --produceruser=<user> ]")
    print("                         [ --producerpass=<password> ]")
    print("                         [ --schema=yes|no ]")
    print("                         [ --schemadb=<url> ]")
    print("                         [ --schemauser=<user> ]")
    print("                         [ --schemapass=<password> ]")
    print("                         [ --registry=yes|no ]")
    print("                         [ --registrydb=<url> ]")
    print("                         [ --registryuser=<user> ]")
    print("                         [ --registrypass=<password> ]")
    print("                         [ --browser=yes|no ]")
    print("                         [ --secure=yes|no ]")
    print("                         [ --port=<port number> ]")
    print("                         [ --output=<filename> ]")
    print("                         [ --silent ]")
    print("")
    print("Options (defaults in brackets):")
    print("  --producerdb   : JDBC url of Producer database (jdbc:mysql://localhost:3306/)")
    print("  --produceruser : Username to access the producer database (producer)")
    print("  --producerpass : Password to access the producer database (rgma)")
    print("  --schema       : Whether to run a schema service (no)")
    print("  --schemadb     : JDBC url of Schema database (jdbc:mysql://localhost:3306/schema)")
    print("  --schemauser   : Username to access the schema database (schema)")
    print("  --schemapass   : Password to access the schema database (rgma)")
    print("  --registry     : Whether to run a registry service (no)")
    print("  --registrydb   : JDBC url of Registry database (jdbc:mysql://localhost:3306/registry)")
    print("  --registryuser : Username to access the registry database (registry)")
    print("  --registrypass : Password to access the registry database (rgma)")
    print("  --browser      : Whether to run a browser service (no)")
    print("  --secure       : Whether to use the HTTPS protocol for contacting services (yes)")
    print("  --port         : Port number the server will run on (8443 for HTTPS, 8080 for HTTP)")
    print("  --output       : Ouput file for SQL commands (rgma-setup-db.sql)")
    print("  --silent       : Suppress all output, using defaults where necessary")
    
def yes_or_no(string):

    string = string.lower().strip()
    return string in ("y", "yes", "true", "t")

def write_header(conf_file):


    conf_file.write("<?xml version=\"1.0\"?>")
    conf_file.write("<RGMA-Server>\n\n")
    
def write_footer(conf_file):

    conf_file.write("</RGMA-Server>\n")
    
def write_server_config(conf_file, dburl, dbuser, dbpassword):

    conf_file.write("<Common>\n")
    conf_file.write("  <accessListReadPeriod>60000</accessListReadPeriod>\n")
    conf_file.write("  <connectionIdFile>WEB-INF/tmp/arch-con-id</connectionIdFile>\n")
    conf_file.write("  <minTermInterval>600</minTermInterval>\n")
    conf_file.write("  <maxTermInterval>129600</maxTermInterval>\n")
    conf_file.write("</Common>\n")
    conf_file.write("\n")
    conf_file.write("<Producer>\n")
    conf_file.write("  <defaultMaxBufferSize>5000000</defaultMaxBufferSize>\n")
    conf_file.write("  <defaultMinRetentionPeriod>0</defaultMinRetentionPeriod>\n")
    conf_file.write("  <cleanupPeriodForMinRetentionTracker>30</cleanupPeriodForMinRetentionTracker>\n")
    conf_file.write("  <maxTupleMemory>35</maxTupleMemory>\n")
    conf_file.write("  <tableAlreadyExistsErrorCode>1050</tableAlreadyExistsErrorCode>\n")
    conf_file.write("  <dbDriver>org.gjt.mm.mysql.Driver</dbDriver>\n")
    conf_file.write("  <defaultDbLocation>%sProducerDB_</defaultDbLocation>\n" % dburl)
    conf_file.write("  <defaultDbUsername>%s</defaultDbUsername>\n" % dbuser)
    conf_file.write("  <defaultDbPassword>%s</defaultDbPassword>\n" % dbpassword)
    conf_file.write("  <dbAdminLocation>%smysql</dbAdminLocation>\n" % dburl)
    conf_file.write("  <dbAdminUsername>%s</dbAdminUsername>\n" % dbuser)
    conf_file.write("  <dbAdminPassword>%s</dbAdminPassword>\n" % dbpassword)
    conf_file.write("  <maxBlockSendSize>500</maxBlockSendSize>\n")
    conf_file.write("  <physicalNamePrefix>_RGMA_</physicalNamePrefix>\n")
    conf_file.write("</Producer>\n")
    conf_file.write("\n")
    conf_file.write("<Consumer>\n")
    conf_file.write("  <diskCachePrefix>/tmp/queue-cache.</diskCachePrefix>\n")
    conf_file.write("  <diskCacheSize>10</diskCacheSize>\n")
    conf_file.write("  <cleanupPeriodForMinRetentionTracker>30</cleanupPeriodForMinRetentionTracker>\n")
    conf_file.write("  <bufferSize>10000</bufferSize>\n")
    conf_file.write("  <streamingPort>8088</streamingPort>\n")
    conf_file.write("  <notifyInterval>10000</notifyInterval>\n")
    conf_file.write("  <defaultQueryTimeOutPeriod>30000</defaultQueryTimeOutPeriod>\n")
    conf_file.write("  <responseTesterTimeOutPeriod>30000</responseTesterTimeOutPeriod>\n")
    conf_file.write("</Consumer>\n")
    conf_file.write("\n")

def write_schema_config(conf_file, dburl, dbuser, dbpassword):
    conf_file.write("<Schema>\n")
    conf_file.write("  <dbDriver>org.gjt.mm.mysql.Driver</dbDriver>\n")
    conf_file.write("  <dbLocation>%s</dbLocation>\n" % dburl)
    conf_file.write("  <dbUsername>%s</dbUsername>\n" % dbuser)
    conf_file.write("  <dbPassword>%s</dbPassword>\n" % dbpassword)
    conf_file.write("</Schema>\n")
    conf_file.write("\n")
    
def write_registry_config(conf_file, dburl, dbuser, dbpassword):
    conf_file.write("<Registry>\n")
    conf_file.write("  <dbDriver>org.gjt.mm.mysql.Driver</dbDriver>\n")
    conf_file.write("  <dbLocation>%s</dbLocation>\n" % dburl)
    conf_file.write("  <dbUsername>%s</dbUsername>\n" % dbuser)
    conf_file.write("  <dbPassword>%s</dbPassword>\n" % dbpassword)
    conf_file.write("  <replicationPeriod>30000</replicationPeriod>\n")
    conf_file.write("  <notifyInterval>30000</notifyInterval>\n")
    conf_file.write("  <duplicateKeyErrorCode>1062</duplicateKeyErrorCode>\n")
    conf_file.write("</Registry>\n")
    conf_file.write("\n")
    
def write_browser_config(conf_file):
    conf_file.write("<Browser>\n")
    conf_file.write("</Browser>\n")
    conf_file.write("\n")

###########################
# Main script starts here #
###########################

# Check RGMA_HOME is set

try:
    rgma_home = os.environ['RGMA_HOME']
except KeyError:
    try:
        rgma_home = os.environ['GLITE_LOCATION']
    except KeyError:
        print "ERROR: Environment variable RGMA_HOME or GLITE_LOCATION must be set"
        sys.exit(1)
  
# Get hostname

hostname = socket.getfqdn();

# Parse command line options

known_options = ["help", "version", \
                 "schema=","registry=","browser=", "output=", \
                 "schemadb=", "schemauser=", "schemapass=", \
                 "registrydb=", "registryuser=", "registrypass=", \
                 "producerdb=", "produceruser=", "producerpass=", \
                 "secure=","port=","silent"]
try:
    options, args = getopt.getopt(sys.argv[1:], "", known_options)
except getopt.GetoptError:
    print("ERROR: parsing command line options\n")
    usage()
    sys.exit(1)

for arg in args:
    print("WARNING: unrecognised command line option '%s' ignored" % arg)

# Set up defaults

schema = False
registry = False
browser = False

output_file = os.path.join(rgma_home, "etc/rgma-server/rgma_sql_conf.sql")
protocol = "https"
port = None
silent = False
install_direct = False

db_admin_user = "root"
db_admin_password = None

monbox_db = "jdbc:mysql://localhost:3306/"
monbox_user = "producer"
monbox_password = "rgma"
monbox_admin_user = None
monbox_admin_password = None
schema_db = "jdbc:mysql://localhost:3306/schema"
schema_user = "schema"
schema_password = "rgma"
schema_admin_user = None
schema_admin_password = None
registry_db = "jdbc:mysql://localhost:3306/registry"
registry_user = "registry"
registry_password = "rgma"
registry_admin_user = None
registry_admin_password = None

for option, value in options:
    if option == "--help":
        usage()
        sys.exit(0)
    elif option == "--version":
        version()
        sys.exit(0)
    elif option == "--producerdb":
        if value[-1:] == "/":
            monbox_db = value
        else:
            monbox_db = value + "/"
    elif option == "--produceruser":
        monbox_user = value
    elif option == "--producerpass":
        monbox_password = value
    elif option == "--schema":
        schema = yes_or_no(value)
    elif option == "--schemadb":
        schema_db = value
    elif option == "--schemauser":
        schema_user = value
    elif option == "--schemapass":
        schema_password = value
    elif option == "--registry":
        registry = yes_or_no(value)
    elif option == "--registrydb":
        registry_db = value
    elif option == "--registryuser":
        registry_user = value
    elif option == "--registrypass":
        registry_password = value
    elif option == "--browser":
        browser = yes_or_no(value)
    elif option == "--output":
        output_file = value
    elif option == "--secure":
        if yes_or_no(value):
            protocol = "https"
        else:
            protocol = "http"
    elif option == "--port":
        port = value
    elif option == "--silent":
        silent = True

if port == None:
    if protocol == "https":
        port = 8443
    else:
        port = 8080

if monbox_admin_user == None:
    monbox_admin_user = db_admin_user
if monbox_admin_password == None:
    monbox_admin_password = db_admin_password

if schema_admin_user == None:
    schema_admin_user = db_admin_user
if schema_admin_password == None:
    schema_admin_password = db_admin_password

if registry_admin_user == None:
    registry_admin_user = db_admin_user
if registry_admin_password == None:
    registry_admin_password = db_admin_password

# Welcome

if not silent:
    print
    print "Setting up an R-GMA Server..."
    print

# Write config file

conf_filename = os.path.join(rgma_home, "etc/rgma-server/rgma-server.conf")
conf_file = open(conf_filename, "w")

write_header(conf_file)

if not silent:
    print "  - Configuring Consumer and Producer services"

write_server_config(conf_file, monbox_db, monbox_user, monbox_password)

if schema:
    if not silent:
        print "  - Configuring a Schema service"
    write_schema_config(conf_file, schema_db, schema_user, schema_password)
else:
    if not silent:
        print "  - Not configuring a Schema service"
    
if registry:
    if not silent:
        print "  - Configuring a Registry service"
    write_registry_config(conf_file, registry_db, registry_user, registry_password)
else:
    if not silent:
        print "  - Not configuring a Registry service"

if browser:
    if not silent:
        print "  - Configuring a Browser service"
    write_browser_config(conf_file)
else: 
    if not silent:
        print "  - Not configuring a Browser service"

write_footer(conf_file)
conf_file.close()

if not silent:
    print
    print "  Wrote configuration to: %s" % conf_filename
    print

# Set up databases

args = " --db-admin-user %s --monbox-admin-user %s --schema-admin-user %s --registry-admin-user %s" \
       % (db_admin_user, monbox_admin_user, schema_admin_user, registry_admin_user)

if db_admin_password != None:
    args += " --db-admin-password %s" % db_admin_password
if monbox_admin_password != None:
    args += " --monbox-admin-password %s" % monbox_admin_password
if schema_admin_password != None:
    args += " --schema-admin-password %s" % schema_admin_password
if registry_admin_password != None:
    args += " --registry-admin-password %s" % registry_admin_password

if install_direct:
    args += " --install"
else:
    args += " --output %s" % output_file

if silent:
    args += " --silent"
    
setup_db_script = os.path.join(rgma_home, "libexec/rgma-server/rgma-setup-db.py") + args
stdout = os.system(setup_db_script)

# Install schema files

if schema:

  temp_filename = os.path.join(rgma_home, "etc/rgma-server/rgma_temp.sql")
  schema_dir = os.path.join(rgma_home, "share/rgma-server/schemas")
  try:
    schema_files = os.listdir(schema_dir)
  except:
    schema_files = []
  
  if len(schema_files) > 0:

      if not silent:
          print
          print "  Processing %i schema files from %s" % (len(schema_files), schema_dir)

      if install_direct:
          args = " --install --user %s" % schema_admin_user
          if schema_admin_password != None:
              args += " --password %s" % schema_admin_password
      else:
          args = " --sql"

      args += " %s/*" % schema_dir

      install_schema_script = os.path.join(rgma_home, "libexec/rgma-server/rgma-schema-tool.py") + args
    
      if install_direct:
          os.system(install_schema_script)
      else:
          stdin, stdout = os.popen2(install_schema_script)
          sql_file = open(output_file, "a")
          sql_lines = stdout.readlines()
    
          for line in sql_lines:
              sql_file.write(line)
   
          sql_file.close()
          stdout.close()

# Configure rgma-servicetool/services to include R-GMA services

services_dir = os.path.join(rgma_home, "etc/rgma-servicetool/services")
config_dir = os.path.join(rgma_home, "share/rgma-server/servicetool")

basic_services = ("RGMA.SecondaryProducerService", "RGMA.ConsumerService", \
                  "RGMA.OnDemandProducerService", "RGMA.PrimaryProducerService")
old_services = ("RGMA.ArchiverService", "RGMA.CanonicalProducerService", \
                "RGMA.StreamProducerService", "RGMA.LatestProducerService", \
                "RGMA.DBProducerService")

schema_services = ("RGMA.SchemaService",)
registry_services = ("RGMA.RegistryService",)
browser_services = ("RGMA.BrowserService",)

all_services = basic_services + schema_services + registry_services + browser_services + old_services

for service in all_services:
    service_file = os.path.join(services_dir, service)
    if os.path.exists(service_file):
        os.remove(service_file);
    
service_files=[]
for service in basic_services:
    service_files.append(service)

if registry:
    for service in registry_services:
        service_files.append(service)

if browser:
    for service in browser_services:
        service_files.append(service)

if schema:
    for service in schema_services:
        service_files.append(service)

if os.path.exists(services_dir):
    if not silent:
        print
        print  "  Configuring the service publisher tool for R-GMA services"
        print
        
    for file in service_files:
        service_file = open(os.path.join(config_dir, file), "r")
        installed_file_name = os.path.join(services_dir, file)
        installed_file = open(installed_file_name, "w")
      
        lines = service_file.readlines()
        for line in lines:
            line = line.replace("@ENDPOINT_PREFIX@", "%s://%s:%s" % (protocol, hostname, port))            
            line = line.replace("@HOSTNAME@", hostname)
            installed_file.write(line)

        service_file.close()
        installed_file.close()

        os.chmod(installed_file_name, 0644)

        if not silent:
            print "  - Copied %s to %s" % (file, services_dir)
else:
    if not silent:
        print
        print "R-GMA servicetool was not installed - service data will not be published"
        print
        print "  - To publish R-GMA services, please install the R-GMA servicetool"
        print "    and re-run this setup script"

# Finished

if not silent:
    if not install_direct:
        print
        print "Finished - Please initialise the R-GMA databases using the following command:"
        print
        print "  mysql -u root -p < " + output_file
        print
        print "Then restart your Tomcat server"
        print
    else:
        print
        print "Finished - Please restart your Tomcat server"
        print
